﻿:Class SALT
⍝ Simple APL Library Toolkit for V11

:Field readonly Public Shared Version←1.17
⍝ Version 1.17, 2007/9/13 added support for svn
⍝ Version 1.16, 2007/9/05 Changed ext ns name to SALT_Data, extension handling, Settings, List
⍝ Version 1.15, 2007/7/23 Changed external namespace name to ÛÁÐÝ_Data
⍝ Version 1.14, 2007/6/15 Noprompt for Save
⍝ Version 1.13, 2007/6/07 Added SALTEXT
⍝ Version 1.12, 2007/5/17 Changed Save to accept also a single string
⍝ Version 1.11, 2007/3/19 Added -collapse to ForgetVersions
⍝ Version 1.10, 2007/3/17 Modified List to behave more consistently
⍝ Version 1.09, 2007/3/16 Modify compare to compare with LAST version
⍝ Version 1.08, 2007/2/22 Load to bring in required material
⍝ Version 1.07, 2007/2/16 Changed ForgetBackups to RemoveVersions
⍝ Version 1.06, 2007/2/11 Change version numbering, removed copy feature from Save
⍝ Version 1.05, 2007/1/28 Add _TRAP, allow to compare using APL
⍝ Version 1.04, 2007/1/18 Allow to save permanently using a table driven technique
⍝ Version 1.03, 2007/1/16 Use multiple work folders
⍝ Version 1.02, 2007/1/13 Modified comments, removed ⋄s, added the EditorCmd field

:Include SALTUtils

⎕USING←0⍴⊂''  ⋄ ⎕ML←3 ⋄ ⎕IO←1 ⋄ ⎕PW←255

⍝ --- Private fields

:Field Private Shared _TRAP←(709+⍳5)'C' '(↑⎕dm)⎕signal⎕en'
:Field Private Shared CR←⎕av[4]
:Field Private Shared WIN
:Field Public  Shared FS←(1+WIN←↑'Windows'⍷1⊃'.'⎕wg'APLVersion')⌷'/\'
:Field Private Shared LINDEL←(~WIN)↓13 10
:Field Private Shared LU←⊃'abcdefghijklmnopqrstuvwxyzàáâãåèéêëòóôõöøùúûäæü' 'ABCDEFGHIJKLMNOPQRSTUVWXYZÀÁÂÃÅÈÉÊËÒÓÔÕÖØÙÚÛÄÆÜ'
:Field ReadOnly Shared Public SALTEXT←'.dyalog'

:Field Shared SettingsTable
SettingsTable←0 5⍴'' ⍝ name; description; registry name; default; value
SettingsTable⍪←'compare;the comparison program to use;CompareCMD;;'splitOn';' ⍝ e.g. [ProgramFiles]Compare It!\wincmp3
SettingsTable⍪←'editor;the editor program to use;EditorCMD;notepad;'splitOn';'
⍝ WorkFolders are locations where files are searched - their existence is not challenged
SettingsTable⍪←'workdir⍟the list of folders to use separated by ;⍟SourceFolder⍟[Dyalog]\Classes⍟'splitOn'⍟'
SettingsTable⍪←'owpfk;special PFKey;owpfk;;' splitOn';'

⍝ --- Utilities

if←/⍨                 ⍝ as in '→0 if condition'
dotVer←{(0<⍵)/'.',⍕⍵} ⍝ >0 versions appear with a dot

⍝ --- Public methods

∇ Reboot
:Access Shared Public
⎕SE.Dyalog.Callbacks.BootSALT
∇

∇ ref←Boot file;f;txt;method;cmd;data;folder;nf;wf;leave;sfile;line
:Access Shared Public

:If isHelp file←fixFsep,file
ref←'Boot filename' ''
ref,←'Boot from a file containing instructions' 'Modifiers accepted:' ''
ref←⊃ref,⊂'(none)'
→0
:EndIf

:For i :In ⍳nf←⍴wf←getSetting'workdir'
sfile←(i⊃wf)ClassFolder file
folder←(1⊃FS splitLast sfile),FS
sfile←sfile{⍺,(⍵≢(-⍴,⍵)↑⍺)/⍵}'.dyapp' ⍝ add extension if absent
:Trap leave←0
txt←⎕SE.UnicodeFile.ReadNestedText sfile ⍝ read file as list of strings
leave←1 ⍝ we found the file, exit the for loop
:Else
(↑⎕DM)⎕SIGNAL(i=nf)⍴⎕EN
:EndTrap
:If leave ⋄ :Leave ⋄ :EndIf
:EndFor

:For line :In txt                   ⍝ read each command
(cmd data)←line splitOn1st' '
:Select lCase cmd
:Case 'load'
:If ~FS∊data
data←folder,data
:EndIf ⍝ Default folder is where "dyalogapp" file was
:Trap 0
'Loaded: ',⍕Load data
:Else
'Load failed for: ',data ⋄ →0
:EndTrap
:Case 'run'
#.⍎data
:CaseList '//' ''
:Else
'Invalid command, ignored: ',line
:EndSelect
:EndFor
∇

∇ ref←New class;hasargs;args;⍙VERSION;cmd;⎕TRAP
:Access Shared Public

:If isHelp class
ref←'New classname [constructor arguments]' ''
ref,←⊂'Create an instance of class without naming it in the workspace'
ref,←'Modifiers accepted:' ''
ref←⊃ref,⊂'-Version=       Select specific version'
→0
:EndIf

:If hasargs←args←~isChar class ⋄ (class args)←class ⋄ :EndIf

⎕TRAP←_TRAP
class←1↓'-version= -= '∆parse class
cmd←class,' -noname',(0≢⍙VERSION)/' -version=',⍙VERSION
ref←#.⎕NEW(Load cmd),hasargs/⊂args
∇

∇  {ref}←{protect}Load class;src;file;⍙TARGET;⍙NONAME;⍙NOLINK;⍙SOURCE;⍙VERSION;src;name;c;w;t;v;folder;ext;target;allpaths;ver;nf;i;⎕TRAP;lastv;req;needed;line;cname
:Access Shared Public
:If isHelp class
ref←'Load [path]classname' '' 'Load a class in the workspace'
ref,←'Modifiers accepted:' ''
ref,←⊂'-Target=Namespace      Specifies target namespace for load'
ref,←⊂'-NoName                Only return ref, do not create name'
ref,←⊂'-NoLink                Do not "link" loaded class to source file'
ref,←⊂'-Version=              Load specific version'
ref,←⊂'-Source                Return the source'
ref←⊃ref ⋄ →0
:EndIf

⎕TRAP←_TRAP
class←fixFsep 1↓'-target= -noname -nolink -source -version= -= '∆parse class

allpaths←∪ClassFolder∘class¨(getSetting'workdir')∪⊂'[Dyalog]\Classes'
lastv←0≡⍙VERSION ⍝ is the last version wanted?
ver←dotVer ⍙VERSION←⌊|num ⍙VERSION

⍝ Locate source
src←0
:For i :In ⍳nf←⍴allpaths ⍝ search each folder
:If 0<⍴v←↑ListVersions/folder cname←FS splitLast class←remExt i⊃allpaths
:If lastv ⋄ ver←dotVer v←⍙VERSION←⌈/v
:Else ⋄ 'version not found'⎕SIGNAL 22 if~⍙VERSION∊v
:EndIf
src←⎕SE.UnicodeFile.ReadNestedText name←class,ver,SALTEXT
:Leave
:EndIf
:EndFor
⎕SIGNAL 22 if src≡0 ⍝ file found?

target←#
:If ~(⊂⍙TARGET)∊0,⊂,'#'
'Target namespace not found'⎕SIGNAL 11 if(9=#.⎕NC ⍙TARGET)⍱⍙TARGET≡'⎕SE'
target←#⍎⍙TARGET
:EndIf

:If ⍙SOURCE ⋄ ref←src ⋄ →0 ⋄ :EndIf ⍝ exit returning source

⍝ We are now ready to FIX the class
⍝ Some classes may require other namespaces to be present,
⍝ if so, we need to bring them in BEFORE we fix the class:
'protect'Default 0
req←'⍝∇:require '
:For line :In (∨/¨'⍝∇:'∘⍷¨src)/src
:If 0<⍴needed←rlb(¯1+(⍴req)+1⍳⍨req⍷lCase line)↓line
1 Load(folder{'='=↑p←(¯1+⍵⍳'⍝')↑⍵:⍺,1↓p ⋄ p}needed),' -tar=',⍕target  ⍝ same location
:EndIf
:EndFor

:Trap 0
:If protect ⍝ we need to find if the name is available
c←':class ' ':interface ' ':namespace '
:For w :In src
:If 4>i←1⍳⍨↑¨c⍷¨⊂lCase t←rlb w
cname←(t⍳' ')↑t←(i⊃⍴¨c)↓t ⋄ :Leave
:EndIf
:EndFor
:AndIf 9=target.⎕NC cname
ref←target.⍎cname
:Else
ref←(~⍙NONAME)target.⎕FIX src
:EndIf
:Else
⎕PW←255
w←'⎕SE'⎕WG'StatusWindow' ⋄ w.⎕WX←1
⎕←'Could not ⎕FIX:'
⎕←⊃w.Text
→0
:EndTrap

:If ~⍙NOLINK ⋄ SetDelta ref name ⍙VERSION((~⍙NONAME)/⍕ref) ⋄ :EndIf
∇

∇ tgt←Save arg;⍙VERSION;⍙NOPROMPT;savename;cmd;folder;name;filename;version;named;ref;t;prev;v;i;r;delims;⎕TRAP;there
:Access Shared Public

:If isHelp arg
r←'Save ''class filename -modifiers''' ''
r,←'Save class in a specific file' '' 'Modifiers:' ''
r,←⊂'-Version=       Version number to save'
r,←⊂'-Noprompt       Do not prompt for confirmation'
tgt←⊃r ⋄ →0
:EndIf

⍝ We accept strings containing a class name AND a (ref string) argument
:If isChar arg
t←' '⍳⍨arg←rlb arg ⋄ ref←⍎(↑⎕NSI),'.',t↑arg ⋄ cmd←t↓arg
:Else
(ref cmd)←2↑arg,⊂''
:EndIf

⎕TRAP←_TRAP
savename←remExt fixFsep 1↓'-version[=] -noprompt -= '∆parse cmd
⍙VERSION←⌊|num ⍙VERSION ⍝ -ver alone will yield 1

:If named←0≠⍴savename ⍝ file name specified
filename←(↑getSetting'workdir')ClassFolder savename
:Else                 ⍝ file name NOT specified
'Save cannot be used on SALT itself'⎕SIGNAL({0::0 ⋄ ⎕THIS≡⍵}ref)/11
'Ref does not point to a SALTed namespace'⎕SIGNAL(9≠ref.⎕NC'SALT_Data')/11
filename←ref.SALT_Data.({(-(1+×Version)⍳⍨+\'.'=⌽⍵)↓⍵}SourceFile)
:If ⍙VERSION≡0 ⋄ ⍙VERSION←{⍵+×⍵}ref.SALT_Data.Version ⋄ :EndIf
:EndIf

(folder name)←FS splitLast filename
v←folder ListVersions name
:If ⍙NOPROMPT<there←⍙VERSION∊v
→0 if 1≠msgBox('Save ',filename)('Confirm overwrite of version ',(↑fmtVersion ⍙VERSION),'?')'Warn'
:EndIf
⍝ The file may not be there but the version to use in less than the highest version
:If ⍙NOPROMPT<there<⍙VERSION<t←⌈/0,v ⍝ Version is LESS than highest version number
→0 if 1≠msgBox('Save ',filename)('Note: Highest existing version is number ',(↑fmtVersion t),'. Proceed?')'Warn'
:EndIf

tgt←folder,FS,name,(dotVer ⍙VERSION),SALTEXT

:Trap 0
⎕SE.UnicodeFile.Write tgt(⎕SRC ref)''LINDEL
SetDelta ref tgt ⍙VERSION({0::'' ⋄ ⍵.SALT_Data.GlobalName}ref)
:Else
('Unable to create file ',tgt)⎕SIGNAL 22
:EndTrap
∇

∇ r←RemoveVersions name;⍙VERSION;⍙ALL;folder;list;v;⎕TRAP;fc;t;cf;⍙COLLAPSE;b
:Access Shared Public

:If isHelp name
r←'Forget backups (usually preceeding a particular version).' '' 'Modifiers:' ''
r,←⊂'-Version=   Specific version(s) to delete'
r,←⊂'-Collapse   Merge LAST deleted version into one'
r,←⊂'-All        Forget ALL backups'
r←⊃r ⋄ →0
:EndIf
⎕TRAP←_TRAP
name←fixFsep 1↓'-version= -all -collapse -= '∆parse name
v←+/0≢¨⍙ALL ⍙VERSION
('You must specify ',(5×2>v)↓'only one of VERSION or ALL')⎕SIGNAL 11 if 1≠v
fc←∊ ⍝ versions must be in the set specified
⍙COLLAPSE∧←∨/2↓b←'<≤≥>'=t←1↑⍙VERSION ⍝ Collapse only with > & ≥
:If cf←∨/b ⋄ fc←⍎t ⋄ ⍙VERSION↓⍨←1 ⋄ :EndIf
'too many versions to compare'⎕SIGNAL 11 if cf∧1≠⍴⍙VERSION←num ⍙VERSION

folder←2⊃(name,'*')locateIn getSetting'workdir'
name←2⊃FS splitLast name
v←folder ListVersions name
'Object not found'⎕SIGNAL(⍴v)↓22 ⍝ Single object not a folder

:If ⍙ALL
⍙VERSION←v
:Else
⍙VERSION←(v fc ⍙VERSION)/v
'NO version found to delete'⎕SIGNAL 22 if 0∊⍴⍙VERSION
:EndIf

Forget folder name ⍙VERSION(⍙COLLAPSE∨0∊⍴v~⍙VERSION)
∇

∇ r←Explore name;list;i;files;cmd;m;⍙MODE;⍙USE;⍙PERMANENT;folder;path;⎕TRAP;v
:Access Shared Public

:If isHelp name
r←'Explore name' ''
r,←'Opens external editor on source files and explorer on folders.' ''
r,←⊂'-Mode=         to specify Normal or Maximised window size'
r,←⊂'-Use=          to specify the EXE to use'
r,←⊂'-Permanent     to remember the EXE to use in the future'
r←⊃r ⋄ →0
:EndIf

⎕TRAP←_TRAP
name←fixFsep 1↓'-Mode=normal maximised -= -use= -permanent'∆parse name
r←⍬
⍙MODE ∆default←'Normal'
⍙USE ∆default←getSetting'editor'
:If ⍙PERMANENT ⋄ 'editor'saveSettings ⍙USE ⋄ :EndIf ⍝ replace in registry
cmd←''
:For folder :In getSetting'workdir'
path←folder ClassFolder name
:If isDir path
⎕SIGNAL WIN↓16 ⋄ cmd←'"explorer.exe "'
:ElseIf 0<⍴v←↑ListVersions/FS splitLast path
path←m,path,(dotVer⌈/v),SALTEXT,m←WIN/'"'
cmd←' ',⍙USE,' '
:EndIf
:If cmd∨.≠' '
⎕CMD ⍙MODE{⍵ ⍺}⍣WIN+1⌽cmd,path ⋄ →0
:EndIf
:EndFor
'Object not found'⎕SIGNAL 22
∇

∇ r←Compare name;⍙VERSION;list;i;files;cmd;folder;v;new;old;⍙USE;⍙PERMANENT;n;⍙ZONE;⍙TRIM;text1;text2;names;s;nf;fmt;⎕TRAP
:Access Shared Public

:If isHelp name
r←'Compare file1 [file2]' '' 'By default, compares current to previous.' '' 'Modifiers:' ''
r,←⊂'-Version=n       compare version n to current'
r,←⊂'-Version=n1 n2   compare version n1 to n2'
r,←⊂'-Use=            to specify the EXE to use'
r,←⊂'-Permanent       to remember the EXE to use in the future'
r,←'' 'If no external program name is used the following modifiers are accepted:'
r,←⊂'-Zone=           the zone to show around differences (default 2)'
r,←⊂'-Trim            trim ends prior to compare'
r←⊃r ⋄ →0
:EndIf

⎕TRAP←_TRAP
names←s splitOn 1↑s←fixFsep'-version= -use= -permanent -zone= -trim'∆parse,⊃name
⍙USE ∆default←getSetting'compare'
⍙VERSION←⍙VERSION ∆default 0
'too many files/versions'⎕SIGNAL 11 if∨/2<(⍴names),⍴,⍙VERSION

:If ⍙PERMANENT ⋄ 'compare'saveSettings ⍙USE ⋄ :EndIf ⍝ replace in registry

→0↓⍨nf←⍴files←names
fmt←{folder,FS,name,((0<⍵)/'.',⍕⍵),SALTEXT}
⍝ Find the file(s) and its versions
:For i :In ⍳nf
name←i⊃names
(folder name)←FS splitLast name
folder←(getSetting'workdir')ClassFolder¨⊂folder
n←1⍳⍨×,⊃⍴¨v←folder ListVersions¨⊂name ⍝ look in each folder
(name,' not found')⎕SIGNAL 22 if n>⍴v
(folder v)←n⊃¨folder v
:If 1=nf  ⍝ 1 file?
(old new)←2↑⍙VERSION,⌈/v
⍝ The are 2 possibilities:
⍝ 1: no version was specified (⍙VERSION≡0) and we compare the last 2 versions:
:If ⍙VERSION≡0 ⋄ old←⌈/¯1,v~new ⋄ :EndIf
⍝ 2: ⍙VERSION=x was specified and we compare x (possibly 0) & last
('Version not found for ',name)⎕SIGNAL(∧/(old,new)∊v)↓22
('Versions are the same')⎕SIGNAL(old=new)⍴11
files←fmt¨old new
:Else
:If ⍙VERSION≡0 ⋄ new←⌈/v
:Else ⋄ new←i⊃2⍴⍙VERSION ⋄ :EndIf
('Version not found for ',name)⎕SIGNAL(new∊v)↓22
files[i]←⊂fmt new
:EndIf
:EndFor

:If ('apl'≡lCase ⍙USE)∨' '∧.=⍙USE
:If 9≠⎕NC'compare' ⋄ Load'Utils\Compare -target=',⍕⎕THIS ⋄ :EndIf
compare.ZONE←⍙ZONE ∆default 2
compare.TRIM←⍙TRIM
text1←⎕SE.UnicodeFile.ReadNestedText 1⊃files
text2←⎕SE.UnicodeFile.ReadNestedText 2⊃files
r←'Comparing ',(1⊃files),CR,'     with ',2⊃files
r←r,,CR,text1 compare.compecv text2
:Else
cmd←{(⍴⍵)>p←⍵⍳']':(p↑⍵)ClassFolder p↓⍵ ⋄ ⍵}⍙USE
:If WIN ⋄ files←1⌽¨'""'∘,¨files ⋄ :EndIf
⍝ If the command contains % file placeholder use them
:If 2=⍴i←(cmd='%')/⍳⍴cmd
cmd[i]←files ⋄ cmd←∊cmd
:Else ⍝ otherwise append the names to the command
cmd←1↓⍕cmd files ⍝ remove leading space from ⍕
:EndIf
⎕CMD cmd'Normal'
r←0 0⍴''
:EndIf
∇

⍝ List command
⍝ Specs:
⍝ List [\root]path[\filter]
⍝ If a root is supplied we use that instead of the current workdir setting
⍝ If a filter is given then path is a folder and the filter is used for filenames
⍝ and any recursize call will include the same filter
⍝ If no filter is given then path may mean a folder with filter * or a folder followed by a filename

∇ r←List arg;folder;d;⍙FULL;⍙RECURSIVE;⍙RAW;dirs;i;recur;files;ext;ver;hasver;filter;f;m;wf;⎕TRAP;es;t;⍙VERSIONS;f0;ls;⍙FOLDERS;b;isfilter;rpf
:Access Shared Public

:If isHelp arg
r←'List pathname' '' 'Modifiers accepted:' ''
r,←⊂'-Full[=1∣2]      1 Show full pathnames below first folder found; 2 returns "rooted" names.'
r,←⊂'-Recursive       Recurse through folders'
r,←⊂'-Versions        List versions'
r,←⊂'-Folders         Only list folders'
r,←⊂'-Raw             Return unformatted date and version numbers'
r←⊃r ⋄ →0
:EndIf

ls←{⍵≡'*':'a'Dir ⍺,FS,⍵ ⋄ ('ad'Dir ⍺,FS,'*'),¨'af'Dir ⍺,FS,⍵,SALTEXT} ⍝ list fn

:If recur←2=≡arg
folder filter ⍙FULL ⍙RECURSIVE ⍙VERSIONS ⍙RAW ⍙FOLDERS←arg ⍝ Internal call
files←folder ls filter
:Else
⍝ Initial call goes thru here. We accept (folders\)*(name∣filter)
⎕TRAP←_TRAP
f←fixFsep 1↓'-full[∊]12 -recursive -versions -folders -raw -= '∆parse arg
⍙FULL←num ⍙FULL
(folder filter)←FS splitLast rpf←remExt(-FS=¯1↑f)↓f ⍝ drop last \ if any

⍝ The arg can be 1:/.../dir (all is listed), 2:/.../name (name only) 3:/.../nam* (all nam... only)
⍝ The easiest way to do it is to list the folder and look for filter as a dir

isfilter←∨/'?*'∊filter ⋄ wf←getSetting'workdir'
:If FS=(1+WIN×2×':'∊2↑rpf)⊃3↑rpf ⍝ rooted?
wf←⊂folder ⋄ filter←rpf←(1+⍴folder)↓rpf
:EndIf
⍝ If a filter is supplied we know the path is in 'folder'. If not we need to figure out
⍝ if 'filter' is a filename (in which case we're all set to proceed) or another folder
⍝ in which case we need to fetch all the files in it.
(f0 folder files)←(rpf,'*')locateIn wf
⍝ If this was a folder we need to call Dir once more to get the files
:If 1∊b←(4⊃files)∊⊂filter ⍝ our name is there?
:AndIf (b⍳1)⊃1⊃files      ⍝ and it's a folder?
t←(0<⍴filter)/FS,filter
files←'a'Dir folder,t,FS,filter←'*'
folder,←(~isfilter)/t
:EndIf
:EndIf

folder,←FS ⋄ es←⊂''
d←↑files ⍝←'a'Dir folder,filter ⍝ dir, ts, size, name
dirs←(⊂'<DIR>'),(es,d⌿⊃[1]files)[;5 1 1 3]
dirs[;2]←folder∘,¨dirs[;2]
dirs←↓dirs
filter,←(0∊⍴filter)/'*'
:If ⍙RECURSIVE∧0<t←⍴dirs ⍝ do NOT enter if t=0
r←⍬ ⋄ ⍙FULL⌈←1
:For i :In ⍳t
r←r,dirs[i],List(i 2⊃dirs)filter ⍙FULL ⍙RECURSIVE ⍙VERSIONS ⍙RAW ⍙FOLDERS
:EndFor
dirs←r
:EndIf
files←(es,(⍙FOLDERS⍱d)⌿⊃[1]files)[;1 5 1 4 3]
files←↓(SALTEXT∘≡¨(-⍴SALTEXT)↑¨files[;2])⌿files
:If 0<1↑⍴t←⊃files
t[;2]←(-⍴SALTEXT)↓¨t[;2] ⋄ files←t
files[;2 3]←t←⊃{∧/⎕D∊⍨⍵↑⍨n←-⊥⍨'.'≠⍵:((n-1)↓⍵)(n↑⍵) ⋄ ⍵''}¨files[;2] ⍝ split version off name
:If 1<↑⍴files
files←files[⍒⊃files[;5];]  ⍝ Desc by timestamp
files←files[⍋⊃files[;2];]  ⍝ Asc by name
:EndIf
:If ⍙VERSIONS=0
files[;3]←es ⋄ t←files[;2]
files←files[t⍳∪t;] ⍝ keep the last (on top) versioned files
:EndIf

files[;2]←folder∘,¨files[;2]
files←↓files
:EndIf

r←files,dirs

:If ~recur ⍝ finalize result
r←⊃r
:Select ↑⍙FULL
:Case 1
r[;2]←(1+⍴f0)↓¨r[;2]
:Case 0
r[;2]←{2⊃FS splitLast ⍵}¨r[;2]
:EndSelect
:If ⍙FOLDERS ⋄ r←r[;,2]
:ElseIf ~⍙RAW
r[;5]←↓fmtDate⊃r[;5]
r[;3]←fmtVersion r[;3]
r←'Type' 'Name' 'Version' 'Size' 'Last Update'⍪r
:EndIf

:EndIf
∇

∇ r←ResetSettings arg;i;s
:Access shared
'unknown setting'⎕SIGNAL 11 if 0∊⍴i←i/⍳⍴i←(arg≡'')∨SettingsTable[;1]∊⊂arg←arg~' '
SettingsTable[i;5]←regSetting¨↓SettingsTable[i;3 4],0 ⍝ cannot be empty
:If 0<⍴s←↑SettingsTable[i←SettingsTable[;1]⍳⊂'owpfk';5]
:AndIf 0 36≠.≤s←↑num s
⎕←'PFkey ',(⍕s),' can be used to overwrite on save'{⍺}((⊂'RL'),'⍝○⎕∪',⊂'EP')⎕PFKEY s
:EndIf
r←0 0⍴''
∇

ResetSettings ''

∇ r←getSetting s
r←↑SettingsTable[SettingsTable[;1]⍳⊂s;5]
:If s≡'workdir' ⋄ r splitOn←';' ⋄ :EndIf
∇

∇ r←Settings arg;pairs;⍙PERMANENT;⍙RESET;vars;names;i;val;add;rem;new;empty;⎕TRAP;k;old;valid
:Access Public Shared
:If isHelp arg
r←'Setting [Name [Value]]' '' 'Returns the [new] settings' 'Modifiers:' ''
r,←⊂'-Permanent      sets permanently'
r,←⊂'-Reset          load permanently saved values'
r,←'' 'The possible settings are:' ''
r,←↓⍕SettingsTable[;1 2]
r←⊃r ⋄ →0
:EndIf
⎕TRAP←_TRAP
:If 1<⍴pairs←'⍟'splitOn⍨'-=⍟ -permanent -reset'∆parse arg
pairs←(1↑pairs),⊂1↓∊' ',¨1↓pairs ⍝ merge last strings
:EndIf
names←SettingsTable[;1]
valid←0<i←names strIndex pairs[1]←lCase¨pairs[1]
'Invalid setting'⎕SIGNAL 11 if valid⍱empty←0∊⍴↑pairs
'-Permanent and -Reset are mutually exclusive'⎕SIGNAL 11 if ⍙PERMANENT∧⍙RESET
pairs[valid/1]←names[valid/i] ⍝ restore to full length if it wasn't
:If ⍙RESET ⋄ ResetSettings 1⊃pairs ⋄ :EndIf

:If 1=⍴pairs
:If empty
r←SettingsTable[;1 5]  ⍝ return all values
:Else
r←SettingsTable[i;1 5] ⍝ return actual value
:EndIf
:Else
val←2⊃pairs
old←(↑SettingsTable[i;5])splitOn';'
:Select ↑pairs
:Case 'workdir'             ⍝ special treatment for folders
new↓⍨←∨/(add rem)←',~'=↑new←val
val←new splitOn';'      ⍝ split names that need to be
:If add ⋄ val←∪val,old  ⍝ side-effect to move ahead
:ElseIf rem ⋄ val←old~val
:EndIf
val←↑{⍺,';',⍵}/val
:Case 'owpfk'
:If val≢old
'invalid PFKey'⎕SIGNAL 11↓⍨((⊂'RL'),'⍝○⎕∪',⊂'EP')setPfkey val
''setPfkey num old ⍝ reset previous key
:EndIf
:EndSelect
SettingsTable[i;5]←⊂val
r←SettingsTable[i;1 5]     ⍝ return effective value
:EndIf

:If ⍙PERMANENT ⋄ saveSettings/r ⋄ :EndIf
:If ~empty ⋄ r←2⊃r ⋄ :EndIf
∇

:EndClass