﻿:Namespace fileUtils
    
⍝ This namespace contains various file utilities. It requires .Net.
    
⍝∇:Require Utils\textUtils
⍝∇:Require Utils\aplUtils
⍝∇:Require Utils\Regex
    
    DefaultExtensions←'dyalog asmx aspx asax apl' ##.textUtils.splitOn ' '
    
    ∇ files showExpr arg;adir;str;file;oname;regex;uf;filedir;⎕PATH;hits;⍙rec;⍙opt;⍙types
    ⍝ This fn will show where Expressions occur in files
     
    ⍝ The left argument to this fn is a string representing a file or a directory
    ⍝ It can include the following switches:
    ⍝  -types=extension   only the files with that extension are scanned (default .dyalog .asmx .aspx .asax and .apl )
    ⍝  -options=I+M+S     to search Insensitive, Singleline or Multiline (the default)
    ⍝  -rec               to search recursively
    ⍝ The right arg is the string to search, a full fledged regular expression, which can include
    ⍝ ⍵ or ⍺ to represent a number of an identifier (see the regex class for details).
    ⍝ Ex: 'C:\Dyalog101\samples\ -types=PHP' showExpr '⎕.*?←⍵' ⍝ look for a ⎕var←number
     
      ⎕PATH←'#.textUtils #.aplUtils'
      :If 82≠⎕DR arg ⋄ (arg filedir ⍙types ⍙opt ⍙rec)←arg,(⍴,arg)↓0 0 ⍬ ⍬ 0
      :Else
          filedir←'-types= -rec -opt∊imsIMS -='∆parse files ⎕THIS
          →0⍴⍨' '∧.=filedir,arg
      :EndIf
     
    ⍝ Validate the argument
      :If 0=⍴oname←(⍙types~0)fileInfo filedir,⍙rec⍴'↓' ⋄ ⎕←'No file found in ',filedir ⋄ →0 ⋄ :EndIf
     
      regex←⎕NEW ##.regexMatch(arg ⍙opt)
      :For file :In oname
          uf←⎕NEW ⎕SE.UnicodeFile file.FullName
          :If '0'≠1↑hits←regex.showMatches uf.Text
              ⎕←'' ⋄ ⎕←file.FullName ⋄ ⎕←hits
          :EndIf
      :EndFor
    ∇
    
    ∇ r←{types}fileInfo name;⎕USING;f;tree
    ⍝ Return file information
      :If 0=⎕NC'types' ⋄ :OrIf 0∊⍴types
          types←'*.'∘,¨DefaultExtensions
      :Else
          types←(types splitOn' ')~⊂''
          types←{'*'=c1←1↑⍵:⍵ ⋄ '.'=c1:'*',⍵ ⋄ '*.',⍵}¨types
      :EndIf
      ⎕USING←'System.IO'
      tree←'↓'∊name ⋄ name~←'↓'
      r←⍬ ⋄ f←⎕NEW FileInfo,⊂⊂name
     
      :If f.Exists
          r←f
      :ElseIf tree
          r←types treefileInfo name
      :ElseIf (f←⎕NEW DirectoryInfo,⊂⊂name).Exists
          r←⊃,/f.GetFiles¨⊂¨types
      :EndIf
    ∇
    
    ∇ r←{types}treefileInfo name;⎕USING;f
    ⍝ Return file information in form of tree
      ⎕USING←'System.IO'
      r←⍬ ⋄ f←name
      :If 9≠⎕NC'name' ⋄ f←⎕NEW DirectoryInfo,⊂⊂name ⋄ :EndIf
      :If f.Exists
          :If 0=⎕NC'types' ⋄ :OrIf 0∊⍴types ⋄ types←'*.'∘,¨DefaultExtensions ⋄ :EndIf
          r←⊃,/f.GetFiles¨⊂¨condEncl types
          :If ×⍴f←f.GetDirectories⊂,'*'
              r←r,⊃,/types∘treefileInfo¨f
          :EndIf
      :EndIf
    ∇
    
    ∇ files replExpr arg;adir;str;file;oname;regex;uf;filedir;⎕PATH;⍙rec;⍙opt;⍙types;re;wh;new;changes
    ⍝ This fn will replace Expressions where they occur in files
     
    ⍝ The left argument to this fn is a string representing a file or a directory
    ⍝ It can include the following switches:
    ⍝  -types=extension   only the files with that extension are scanned (default .dyalog .asmx .aspx .asax and .apl )
    ⍝  -options=I+M+S     to search Insensitive, Singleline or Multiline (the default)
    ⍝  -rec               to search recursively
    ⍝ The right arg is the string to search, a full fledged regular expression, which can include
    ⍝ ⍵ or ⍺ to represent a number of an identifier (see the regex class for details),
    ⍝ followed by another expression to perform replacement. Both are separated by a space.
    ⍝ If the expression include space they must be surrounded by quotes.
    ⍝ Ex: '\temp\ -types=PHP' replExpr '(\w*),(\w*) ''$2 $1''' ⍝ change word1,word2 into work2 word1
     
    ⍝ WARNING: you should take backup copies of your files before using this function on them.
    ⍝ The changes made are irreversible!
     
      ⎕PATH←'#.textUtils #.aplUtils'
      :If 82≠⎕DR arg ⋄ (arg filedir ⍙types ⍙opt ⍙rec)←arg,(⍴,arg)↓0 0 ⍬ ⍬ 0
      :Else
          filedir←'-types= -rec -opt∊imsIMS -='∆parse files ⎕THIS
          →0⍴⍨' '∧.=filedir,arg
          (wh re)←cutString''∆parse arg ⍝ use ∆parse to get rid of quotes (may fail if 2nd arg starts with -)
      :EndIf
     
    ⍝ Validate the argument
      :If 0=⍴,oname←(⍙types~0)fileInfo filedir,⍙rec⍴'↓' ⋄ ⎕←'No file found in ',filedir ⋄ →0 ⋄ :EndIf
      changes←0
      regex←⎕NEW ##.regexMatch(wh ⍙opt)
      :For file :In oname
          uf←⎕NEW ⎕SE.UnicodeFile file.FullName
          :If uf.Text≢new←regex.Replace uf.Text re
              uf.Export ⋄ changes+←1
          :EndIf
      :EndFor
      changes,'changes made'
    ∇
    
    ∇ fromname copyCptFile toname;i;li;⎕IO;from;to;tied;cn;cl
    ⍝ Copy a component file to a new file.
    ⍝ from is tie no or filename.
    ⍝ to is NEW filename to create and copy to
      ⎕IO←1
    ⍝ If from is a filename we attempt to tie exclusively
      :If ~tied←3∊10|⎕DR from←fromname ⋄ from←fromname ⎕FTIE 0 ⋄ :EndIf
    ⍝ If it was shared tied we cannot guaranty integrity
      cn cl←¯1+2↑⎕FSIZE from
     
    ⍝ Create the new file
      to←toname ⎕FCREATE 0
    ⍝ There is no need to set the size under Dyalog
     
    ⍝ Fill first cpts
    ⍝ First the start component
      :If 0<cn ⋄ (cn⍴0)⎕FAPPEND¨to ⋄ :EndIf
      ⎕FDROP to,cn
      0 ⎕FRESIZE to ⍝ cut back the space used
    ⍝ Then fill the rest
      :While cn<cl
          cn+←1
          (⎕FREAD from,cn)⎕FAPPEND to
      :EndWhile
     
    ⍝ Copy the access matrix
      (⎕FRDAC from)⎕FSTAC to
     
    ⍝ Untie the file if it wasn't tied
      ⎕FUNTIE to,tied↓from
    ∇
    
    ∇ r←filenameOf tieno
    ⍝ Return the filename associated with a tie no
      r←⎕FNAMES[⎕FNUMS⍳tieno;] ⍝ bombs if not a valie tie no
    ∇
    
    ∇ p←filePermission cod
    ⍝ Returns a verbose list of permission from the code of
    ⍝ the access matrix of a component file
      p←' read+size tie erase append replace drop hold '
      p←p,'rename rdac,stac rdci resize fhold rdac stac'
      cod←⌽,(14⍴2)⊤cod
      p←1↓¨cod⌿(p=' ')⊂p
    ∇
    
:EndNamespace